@@ -4,16 +4,23 @@ using Gdk;
 using GLib;
 
 namespace FSpot {
-	public class GroupSelector : Bin {
+	public class GroupSelector : Fixed {
 		internal static GType groupSelectorGType;
+
 		int border = 12;
 		int box_spacing = 2;
 		int box_top_padding = 6;
 		public static int MIN_BOX_WIDTH = 20;
+
 		private Glass glass;
 		private Limit min_limit;
 		private Limit max_limit;
 
+#if USE_BUTTONS
+		private Gtk.Button left;
+		private Gtk.Button right;
+#endif
+
 		private Gdk.Window event_window;
 
 		public Gdk.Rectangle background;
@@ -22,12 +29,31 @@ namespace FSpot {
 		int    box_count_max;
 		int [] box_counts = new int [0];
 		Pango.Layout [] tick_layouts;
-
+		bool   has_limits;
+		
 		protected FSpot.GroupAdaptor adaptor;
 		public FSpot.GroupAdaptor Adaptor {
 			set {
+				if (adaptor != null)
+					adaptor.Changed -= HandleAdaptorChanged;
+
 				adaptor = value;
-				
+				has_limits = adaptor is FSpot.ILimitable;				
+				if (has_limits) {
+				    min_limit.SetPosition (0);
+				    max_limit.SetPosition (adaptor.Count () - 1);
+				}
+
+				adaptor.Changed += HandleAdaptorChanged;
+				HandleAdaptorChanged (adaptor);
+			}
+			get {
+				return adaptor;
+			}
+		}
+
+		private void HandleAdaptorChanged (GroupAdaptor adaptor)
+		{
 				int [] box_values = new int [adaptor.Count ()];
 
 				if (tick_layouts != null) {
@@ -47,15 +73,23 @@ namespace FSpot {
 					}
 					i++;
 				}
-				Counts = box_values;
-				min_limit.SetPosition (0);
-				max_limit.SetPosition (adaptor.Count () - 1);
-			}
-			get {
-				return adaptor;
-			}
-		}
 
+				if (glass.Position >= adaptor.Count())
+					glass.SetPosition (adaptor.Count() - 1);
+
+				Counts = box_values;
+
+				if (has_limits) {
+					if (min_limit.Position > adaptor.Count ())
+						min_limit.SetPosition (0);
+				        
+				     
+					if (max_limit.Position > adaptor.Count ())
+						max_limit.SetPosition (adaptor.Count () - 1);
+				}
+				this.QueueDraw ();
+		}
+		
 		private int [] Counts {
 			set {
 				box_count_max = 0;
@@ -142,9 +176,9 @@ namespace FSpot {
 
 			if (glass.IsInside (x, y)) {
 				glass.StartDrag (x, y, args.Time);
-			} else if (min_limit.IsInside (x, y)) {
+			} else if (has_limits && min_limit.IsInside (x, y)) {
 				min_limit.StartDrag (x, y, args.Time);
-			} else if (max_limit.IsInside (x, y)) {
+			} else if (has_limits && max_limit.IsInside (x, y)) {
 				max_limit.StartDrag (x, y, args.Time);
 			} else {
 				int position;
@@ -174,8 +208,8 @@ namespace FSpot {
 
 		public void UpdateLimits () 
 		{
-			if (adaptor != null && min_limit != null && max_limit != null)
-				adaptor.SetLimits (min_limit.Position, max_limit.Position);
+			if (adaptor != null && has_limits && min_limit != null && max_limit != null)
+				((ILimitable)adaptor).SetLimits (min_limit.Position, max_limit.Position);
 		}
 
 		protected override bool OnMotionNotifyEvent (Gdk.EventMotion args) 
@@ -229,6 +263,7 @@ namespace FSpot {
 		{
 			event_window.Dispose ();
 			event_window = null;
+			base.OnUnrealized ();
 		}
 		
 		private Double BoxWidth {
@@ -245,7 +280,8 @@ namespace FSpot {
 			}
 		}
 
-		private int BoxX (int item) {
+		private int BoxX (int item) 
+		{
 			 return scroll_offset + background.X + (int) Math.Round (BoxWidth * item);
 		}
 		
@@ -265,10 +301,11 @@ namespace FSpot {
 		{
 			int total_height = background.Height;
 			int count = item;
-			double percent = box_counts [item] / (double) box_count_max;
+			double percent = box_counts [item] / (double) Math.Max (box_count_max, 1);
 
 			Rectangle box = Rectangle.Zero;
 			box.Height = (int) Math.Round ((total_height - box_top_padding) * percent + 0.5);
+
 			box.Y = background.Y + total_height - box.Height - 1;
 			
 			box.X = BoxX (item);
@@ -473,9 +510,9 @@ namespace FSpot {
 				Rectangle bounds = Bounds ();
 				Requisition requisition = popup_widow.SizeRequest ();
 				popup_widow.Resize  (requisition.Width, requisition.Height);
-				selector.GdkWindow.GetRootOrigin (out x, out y);
+				selector.GdkWindow.GetOrigin (out x, out y);
 				x += bounds.X + (bounds.Width - requisition.Width) / 2;
-				y += bounds.Y;
+				y += bounds.Y - requisition.Height;
 				x = Math.Max (x, 0);
 				x = Math.Min (x, selector.Screen.Width - requisition.Width);
 				popup_widow.Move (x, y);
@@ -672,14 +709,16 @@ namespace FSpot {
 					DrawTick (area, i++);
 			}
 			
-			if (min_limit != null) {
-				min_limit.Draw (args.Area);
+			if (has_limits) {
+				if (min_limit != null) {
+					min_limit.Draw (args.Area);
+				}
+				
+				if (max_limit != null) {
+					max_limit.Draw (args.Area);
+				}
 			}
-			
-			if (max_limit != null) {
-				max_limit.Draw (args.Area);
-			}
-			       
+
 			if (glass != null) {
 				glass.Draw (args.Area);
 			}
@@ -691,6 +730,7 @@ namespace FSpot {
 		{
 			requisition.Width = 500;
 			requisition.Height = 100;
+			base.OnSizeRequested (ref requisition);
 		}
 
 		private int LengendHeight ()
@@ -722,6 +762,11 @@ namespace FSpot {
 
 			if (event_window != null)
 				event_window.MoveResize (alloc.X, alloc.Y, alloc.Width, alloc.Height);
+
+#if USE_BUTTONS
+			if (right.Allocation.X != 10 || (right.Allocation.Y - alloc.Y) != 10)
+				this.Move (right, 10, 10);
+#endif
 		}
 
 		public GroupSelector () : base () 
@@ -734,6 +779,15 @@ namespace FSpot {
 			max_limit = new Limit (this, Limit.LimitType.Max);
 			min_limit.SetPosition (0);
 			max_limit.SetPosition (11);
+
+#if USE_BUTTONS
+			left = new Gtk.Button (Gtk.Stock.GoBack);
+			right = new Gtk.Button (Gtk.Stock.GoForward);
+			this.Put (left, 0, 0);
+			this.Put (right, 100, 0);
+			left.Show ();
+			right.Show ();
+#endif
 		}
 
 		public GroupSelector (IntPtr raw) : base (raw) {}
