@@ -22,43 +22,48 @@ public class MainWindow {
 	}
 
 	TagSelectionWidget tag_selection_widget;
-	[Widget] Gtk.Window main_window;
-	[Widget] VBox left_vbox;
-	[Widget] VBox group_vbox;
-	[Widget] Toolbar main_toolbar;
-	[Widget] ScrolledWindow icon_view_scrolled;
-	[Widget] Box photo_box;
-	[Widget] Notebook view_notebook;
-	[Widget] ScrolledWindow tag_selection_scrolled;
+	[Glade.Widget] Gtk.Window main_window;
+	[Glade.Widget] VBox left_vbox;
+	[Glade.Widget] VBox group_vbox;
+
+	[Glade.Widget] ScrolledWindow icon_view_scrolled;
+	[Glade.Widget] Box photo_box;
+	[Glade.Widget] Notebook view_notebook;
+	[Glade.Widget] ScrolledWindow tag_selection_scrolled;
 
 	//
 	// Menu items
 	//
-	[Widget] MenuItem version_menu_item;
-	[Widget] MenuItem create_version_menu_item, delete_version_menu_item, rename_version_menu_item;
+	[Glade.Widget] MenuItem version_menu_item;
+	[Glade.Widget] MenuItem create_version_menu_item;
+	[Glade.Widget] MenuItem delete_version_menu_item;
+	[Glade.Widget] MenuItem rename_version_menu_item;
 
-	[Widget] MenuItem delete_selected_tag;
-	[Widget] MenuItem edit_selected_tag;
+	[Glade.Widget] MenuItem delete_selected_tag;
+	[Glade.Widget] MenuItem edit_selected_tag;
 
-	[Widget] MenuItem attach_tag_to_selection;
-	[Widget] MenuItem remove_tag_from_selection;
+	[Glade.Widget] MenuItem attach_tag_to_selection;
+	[Glade.Widget] MenuItem remove_tag_from_selection;
 
-	[Widget] MenuItem copy;
-	[Widget] MenuItem rotate_left;
-	[Widget] MenuItem rotate_right;
-	[Widget] MenuItem update_thumbnail;
-	[Widget] MenuItem delete_from_drive;
+	[Glade.Widget] MenuItem copy;
+	[Glade.Widget] MenuItem rotate_left;
+	[Glade.Widget] MenuItem rotate_right;
+	[Glade.Widget] MenuItem update_thumbnail;
+	[Glade.Widget] MenuItem delete_from_drive;
 
-	[Widget] MenuItem set_as_background;
+	[Glade.Widget] MenuItem display_tags_menu_item;
 
-	[Widget] MenuItem attach_tag;
-	[Widget] MenuItem remove_tag;
-	[Widget] MenuItem find_tag;
+	[Glade.Widget] MenuItem set_as_background;
+
+	[Glade.Widget] MenuItem attach_tag;
+	[Glade.Widget] MenuItem remove_tag;
+	[Glade.Widget] MenuItem find_tag;
 
 	PhotoVersionMenu versions_submenu;
 	
 	InfoBox info_box;
-	IconView icon_view;
+	FSpot.InfoDisplay info_display;
+	QueryView icon_view;
 	PhotoView photo_view;
 	PhotoQuery query;
 	FSpot.GroupSelector group_selector;
@@ -109,7 +114,6 @@ public class MainWindow {
 	}
 
 	
-
 	//
 	// Constructor
 	//
@@ -134,11 +138,14 @@ public class MainWindow {
 		Gtk.Drag.DestSet (tag_selection_widget, DestDefaults.All, tag_dest_target_table, 
 				  DragAction.Copy); 
 
+		tag_selection_widget.ButtonPressEvent += new ButtonPressEventHandler (HandleTagSelectionButtonPressEvent);
+
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += new InfoBox.VersionIdChangedHandler (HandleInfoBoxVersionIdChange);
 		left_vbox.PackStart (info_box, false, true, 0);
 		
 		query = new PhotoQuery (db.Photos);
+		query.ItemChanged += HandleQueryItemChanged;
 
 		group_selector = new FSpot.GroupSelector ();
 		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
@@ -151,11 +158,14 @@ public class MainWindow {
 		group_vbox.PackStart (group_selector, false, false, 0);
 		group_vbox.ReorderChild (group_selector, 0);
 		
-		icon_view = new IconView (query);
+		icon_view = new QueryView (query);
 		icon_view_scrolled.Add (icon_view);
 		icon_view.SelectionChanged += new IconView.SelectionChangedHandler (HandleSelectionChanged);
 		icon_view.DoubleClicked += new IconView.DoubleClickedHandler (HandleDoubleClicked);
-		
+		icon_view.GrabFocus ();
+
+		FSpot.PreviewPopup preview = new FSpot.PreviewPopup (icon_view);
+
 		Gtk.Drag.SourceSet (icon_view, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
 				    icon_source_target_table, DragAction.Copy | DragAction.Move);
 		
@@ -179,7 +189,6 @@ public class MainWindow {
 		icon_view.DragMotion += new DragMotionHandler (HandleIconViewDragMotion);
 		icon_view.DragDrop += new DragDropHandler (HandleIconViewDragDrop);
 		icon_view.DragDataReceived += new DragDataReceivedHandler (HandleIconViewDragDataReceived);
-		icon_view.MotionNotifyEvent += new MotionNotifyEventHandler (HandleIconViewMotionNotifyEvent);
 
 		photo_view = new PhotoView (query, db.Photos);
 		photo_box.Add (photo_view);
@@ -254,18 +263,13 @@ public class MainWindow {
 		photo_view.Update ();
 	}
 
-	private void InvalidateViews (int num)
-	{
-		icon_view.InvalidateCell (num);
-		if (num == photo_view.CurrentPhoto)
-			photo_view.Update ();
-	}
-
 	private void UpdateViews (int num)
 	{
 		icon_view.UpdateThumbnail (num);
 		if (num == photo_view.CurrentPhoto)
 			photo_view.Update ();
+		info_box.Update ();
+		UpdateMenus ();
 	}
 		
 	//
@@ -288,10 +292,10 @@ public class MainWindow {
 	// Tag Selection Drag Handlers
 	//
 
-	public void AddTagExtended (Photo photo, Tag [] tags)
+	public void AddTagExtended (int num, Tag [] tags)
 	{
-		photo.AddTag (tags);
-		db.Photos.Commit (photo);
+		query.Photos [num].AddTag (tags);
+		query.Commit (num);
 
 		foreach (Tag t in tags) {
 			Pixbuf icon = null;
@@ -299,7 +303,7 @@ public class MainWindow {
 			if (t.Icon == null) {
 				if (icon == null) {
 					// FIXME this needs a lot more work.
-					Pixbuf tmp = PixbufUtils.LoadAtMaxSize (photo.DefaultVersionPath, 128, 128);
+					Pixbuf tmp = PixbufUtils.LoadAtMaxSize (query.Photos[num].DefaultVersionPath, 128, 128);
 					icon = PixbufUtils.TagIconFromPixbuf (tmp);
 					tmp.Dispose ();
 				}
@@ -310,6 +314,24 @@ public class MainWindow {
 		}
 	}
 
+
+
+	[GLib.ConnectBefore]
+	void HandleTagSelectionButtonPressEvent (object sender, ButtonPressEventArgs args)
+	{
+		if (args.Event.Button == 3)
+		{
+			TreePath path;
+			tag_selection_widget.Selection.UnselectAll ();
+			if (tag_selection_widget.GetPathAtPos ((int)args.Event.X, (int)args.Event.Y, out path)) {
+				tag_selection_widget.Selection.SelectPath (path);
+			}
+			TagPopup popup = new TagPopup ();
+			popup.Activate (args.Event, tag_selection_widget.TagAtPosition ((int)args.Event.X, (int)args.Event.Y));
+			args.RetVal = true;
+		}
+	}
+
 	public void HandleTagSelectionDragMotion (object o, DragMotionArgs args)
 	{
 		TreePath path;
@@ -333,10 +355,7 @@ public class MainWindow {
 		switch (args.Info) {
 		case (uint)TargetType.PhotoList:
 			foreach (int num in SelectedIds ()) {
-				Photo photo = query.Photos [num];
-				
-				AddTagExtended (photo, tags);
-				InvalidateViews (num);
+				AddTagExtended (num, tags);
 			}
 			break;
 		case (uint)TargetType.TagList:
@@ -349,7 +368,8 @@ public class MainWindow {
 				if (photo == null)
 					return;
 				
-				AddTagExtended (photo, tags);
+				// FIXME this should really follow the AddTagsExtended path too
+				photo.AddTag (tags);
 			}
 			InvalidateViews ();
 			break;
@@ -425,6 +445,14 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
+	public void ImportUriList (UriList list) 
+	{
+		ImportCommand command = new ImportCommand (main_window);
+		if (command.ImportFromPaths (db.Photos, list.ToLocalPaths ()) > 0) {
+			UpdateQuery ();
+		}
+	}
+
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
@@ -457,39 +485,13 @@ public class MainWindow {
 				return;
 
 			UriList list = new UriList (args.SelectionData);
-			ImportCommand command = new ImportCommand (main_window);
-			command.ImportFromPaths (db.Photos, list.ToLocalPaths ());
-			UpdateQuery ();
+			ImportUriList (list);
 			break;
 		}
 
 		Gtk.Drag.Finish (args.Context, true, false, args.Time);
 	}
 
-	void HandleIconViewMotionNotifyEvent (object sender, MotionNotifyEventArgs args)
-	{
-		if ((args.Event.State & Gdk.ModifierType.Mod1Mask) == 0) {
-			HideQuickPreview ();
-			return;
-		}
-
-		int x = (int) args.Event.X;
-		int y = (int) args.Event.Y;
-		int cell_num = icon_view.CellAtPosition (x, y);
-		
-		int image_center_x, image_center_y;
-
-		Rectangle bounds = icon_view.CellBounds (cell_num);
-		image_center_x = bounds.X + (bounds.Width / 2);
-		image_center_y = bounds.Y + (bounds.Height / 2);
-
-		image_center_x += (int) args.Event.XRoot - x;
-		image_center_y += (int) args.Event.YRoot - y;
-		
-		ShowQuickPreview (cell_num, image_center_x, image_center_y);
-	}
-
-
 	//
 	// IconView event handlers
 	// 
@@ -500,11 +502,15 @@ public class MainWindow {
 
 		if (selection.Length == 1) {
 			current_photo_idx = selection [0];
+			//photo_view.CurrentPhoto = current_photo_idx;
 			info_box.Photo = CurrentPhoto;
-
+			if (info_display != null) 
+				info_display.Photo = CurrentPhoto;
 		} else { 
 			current_photo_idx = PHOTO_IDX_NONE;
 			info_box.Photo = null;
+			if (info_display != null)
+				info_display.Photo = null;
 		}
 
 		current_photos = selection.Length > 0;
@@ -518,168 +524,6 @@ public class MainWindow {
 		SwitchToPhotoViewMode ();
 	}
 
-
-	int stored_display_width = -1, stored_display_height = -1;
-
-	void GetDisplaySize (out int w, out int h)
-	{
-		if (stored_display_width == -1
-		    || stored_display_height == -1) {
-			Drawable d = (Display.Default.GetScreen (0)).RootWindow;
-			d.GetSize (out stored_display_width, out stored_display_height);
-		}
-
-		w = stored_display_width;
-		h = stored_display_height;
-	}
-
-	Gtk.Window quick_preview = null;
-	Gtk.Image quick_preview_image = null;
-	Gtk.Label quick_preview_label = null;
-	ThumbnailCache quick_preview_cache = new ThumbnailCache (200);
-	int last_motion_item = -1;
-	int last_image_center_x = -1;
-	int last_image_center_y = -1;
-
-	void CreateQuickPreview ()
-	{
-		quick_preview = new Gtk.Window (Gtk.WindowType.Popup);
-		quick_preview.Decorated = false;
-		quick_preview.TransientFor = main_window;
-		quick_preview.TypeHint = Gdk.WindowTypeHint.Dock;
-		quick_preview.CanFocus = false;
-
-		Gtk.Box quick_preview_vbox = new Gtk.VBox ();
-		// FIXME these should just load a specific style.
-		quick_preview.ModifyFg (StateType.Normal, new Gdk.Color (127, 127, 127));
-		quick_preview.ModifyBg (StateType.Normal, new Gdk.Color (0, 0, 0));
-		
-		quick_preview_image = new Gtk.Image ();
-		quick_preview_image.CanFocus = false;
-		quick_preview_vbox.PackStart (quick_preview_image, true, true, 0);
-		
-		quick_preview_label = new Gtk.Label ("");
-		quick_preview_label.CanFocus = false;
-		quick_preview_vbox.PackStart (quick_preview_label, true, false, 0);
-		quick_preview_label.ModifyFg (StateType.Normal, new Gdk.Color (127, 127, 127));
-		quick_preview_label.ModifyBg (StateType.Normal, new Gdk.Color (0, 0, 0));
-
-		quick_preview.SizeAllocated += new SizeAllocatedHandler (QuickPreviewSizeAllocatedHandler);
-		
-		quick_preview.Add (quick_preview_vbox);
-		quick_preview_vbox.ShowAll ();
-	}
-
-	void PositionQuickPreview ()
-	{
-		int display_width, display_height;
-		GetDisplaySize (out display_width, out display_height);
-
-		int preview_width, preview_height;
-		preview_width = quick_preview.Allocation.Width;
-		preview_height = quick_preview.Allocation.Height;
-
-		Gdk.Rectangle bounds = icon_view.CellBounds (last_motion_item);
-
-		// Compute the preview's x-position, ensuring that it doesn't
-		// fall off the edge of the screen
-		int image_x;
-		image_x = last_image_center_x - preview_width/2;
-		if (image_x + preview_width > display_width)
-			image_x += display_width - image_x - preview_width;
-		else if (image_x < 0)
-			image_x = 0;
-
-		// Compute the preview's y-position.  We put the preview above
-		// the image if it will fit there, and below otherwise.
-		int image_y;
-		int margin = bounds.Height / 2 + 5;
-		image_y = last_image_center_y - preview_height - margin;
-		if (image_y < 0)
-			image_y = last_image_center_y + margin;
-
-		quick_preview.Move (image_x, image_y);
-		quick_preview.Show ();
-	}
-
-	void QuickPreviewSizeAllocatedHandler (object o, SizeAllocatedArgs args)
-	{
-		if (last_motion_item < 0
-		    || quick_preview.Allocation.Width <= 1
-		    || quick_preview.Allocation.Height <= 1)
-			return;
-
-		PositionQuickPreview ();
-	}
-
-	void ShowQuickPreview (int motion_item, int image_center_x, int image_center_y)
-	{
-		if (motion_item < 0
-		    || motion_item >= query.Photos.Length
-		    || motion_item == last_motion_item)
-			return;
-
-		Photo photo = query.Photos [motion_item];
-
-		last_motion_item = motion_item;
-		last_image_center_x = image_center_x;
-		last_image_center_y = image_center_y;
-
-		if (quick_preview == null)
-			CreateQuickPreview ();
-
-		//Pixbuf old = quick_preview_image.Pixbuf;
-		quick_preview_image.Pixbuf = QuickPreviewPixbuf (photo);
-		/*
-		if (old != null)
-			old.Dispose ();
-		*/
-
-		quick_preview_label.Text = QuickPreviewLabel (photo);
-		quick_preview.Show ();
-
-		// The quick preview's position is set in the SizeAllocated
-		// handler.  We need to know how big the window is in order
-		// to position it properly.
-	}
-
-	void HideQuickPreview ()
-	{
-		if (quick_preview != null) {
-			quick_preview.Destroy ();
-			quick_preview = null;
-		}
-		last_motion_item = -1;
-	}
-
-	Pixbuf QuickPreviewPixbuf (Photo photo)
-	{
-		string orig_path = photo.DefaultVersionPath;
-
-		Pixbuf pixbuf = quick_preview_cache.GetThumbnailForPath (orig_path);
-		if (pixbuf == null) {
-			int display_width, display_height;
-			GetDisplaySize (out display_width, out display_height);
-
-			// A bizarre pixbuf = hack to try to deal with cinematic displays, etc.
-			int preview_edge = ((display_width + display_height)/2)/3;
-			//pixbuf = PixbufUtils.LoadAtMaxEdgeSize (orig_path, preview_edge);
-			pixbuf = FSpot.PhotoLoader.LoadAtMaxSize (photo, preview_edge, preview_edge);
-			quick_preview_cache.AddThumbnail (orig_path, pixbuf);
-		}
-
-		return pixbuf;
-	}
-
-	string QuickPreviewLabel (Photo photo)
-	{
-		string label = "";
-		if (photo.Description.Length > 0)
-			label = photo.Description + "\n";
-		label += photo.Time.ToString () + "   " + photo.Name;
-		return label;
-	}
-
 	//
 	// PhotoView event handlers.
 	//
@@ -689,6 +533,8 @@ public class MainWindow {
 		current_photos = true;
 		current_photo_idx = photo_view.CurrentPhoto;
 		info_box.Photo = CurrentPhoto;
+		if (info_display != null)
+			info_display.Photo = CurrentPhoto;
 		UpdateMenus ();
 	}
 
@@ -763,10 +609,7 @@ public class MainWindow {
 	public void HandleAttachTagMenuSelected (Tag t) 
 	{
 		foreach (int num in SelectedIds ()) {
-			Photo photo = query.Photos [num];
-			AddTagExtended (photo, new Tag [] {t});
-			
-			InvalidateViews (num);
+			AddTagExtended (num, new Tag [] {t});
 		}
 	}
 	
@@ -778,11 +621,8 @@ public class MainWindow {
 	public void HandleRemoveTagMenuSelected (Tag t)
 	{
 		foreach (int num in SelectedIds ()) {
-			Photo photo = query.Photos [num];
-			photo.RemoveTag (t);
-			db.Photos.Commit (photo);
-			
-			InvalidateViews (num);
+			query.Photos [num].RemoveTag (t);
+			query.Commit (num);
 		}
 	}
 
@@ -793,14 +633,58 @@ public class MainWindow {
 	void HandleImportCommand (object sender, EventArgs e)
 	{
 		ImportCommand command = new ImportCommand (main_window);
-		command.ImportFromFile (db.Photos);
-		UpdateQuery ();
+		if (command.ImportFromFile (db.Photos) > 0) {
+			UpdateQuery ();
+		}
 	}
 
+#if false
 	unsafe void HandlePrintCommand (object sender, EventArgs e)
 	{
 		PrintJob pj = new PrintJob (PrintConfig.Default ());
 		PrintDialog dialog = new PrintDialog (pj, "Print Images", 0);
+
+		PrintContext ctx = pj.Context;
+		double page_width, page_height;
+		pj.GetPageSize (out page_width, out page_height);
+
+		foreach (Photo photo in SelectedPhotos ()) {
+			Print.Beginpage (ctx, "F-Spot "+ photo.DefaultVersionPath);
+			
+			Pixbuf image = FSpot.PhotoLoader.Load (photo);
+
+			bool rotate = false;
+			double width = page_width;
+			double height = page_height;
+			if (image.Width > image.Height) {
+				rotate = true;
+				width = page_height;
+				height = page_width;
+			}
+			
+			double scale = Math.Min (width / image.Width, 
+						  height / image.Height);
+			
+			Gnome.Print.Gsave (ctx);
+
+			if (rotate) {
+				Gnome.Print.Rotate (ctx, 90);
+				Gnome.Print.Translate (ctx, 0, -page_width);
+			}
+			Print.Translate (ctx, 
+					 (width - image.Width * scale) / 2.0, 
+					 (height - image.Height * scale) / 2.0);
+
+			Print.Scale (ctx, image.Width * scale, image.Height * scale);
+
+			Print.Pixbuf (ctx, image);
+			Print.Grestore (ctx);
+
+			//Print.Show (ctx, photo.Description);
+			Print.Showpage (ctx);
+			image.Dispose ();
+		}
+
 		int response = dialog.Run ();
 
 		Console.WriteLine ("response: " + response);
@@ -808,23 +692,6 @@ public class MainWindow {
 		if (response == (int) PrintButtons.Cancel) {
 			dialog.Destroy ();
 		}
-
-		PrintContext ctx = pj.Context;
-		
-		Print.Beginpage (ctx, "Test");
-		
-		Pixbuf image  = new Pixbuf (query.Photos[0].DefaultVersionPath);
-
-		Print.Moveto (ctx, 100, 100);
-		Print.Gsave (ctx);
-		Print.Translate (ctx, 100, 100);
-		Print.Scale (ctx, 100, 100);
-		Print.Pixbuf (ctx, image);
-		Print.Grestore (ctx);
-
-		Print.Show (ctx, "testing");
-		Print.Showpage (ctx);
-		
 		pj.Close ();
 
 		switch (response) {
@@ -838,8 +705,44 @@ public class MainWindow {
 
 		dialog.Destroy ();
 	}
+#else
+	unsafe void HandlePrintCommand (object sender, EventArgs e)
+	{
+		FSpot.PrintDialog dialog = new FSpot.PrintDialog (SelectedPhotos ());
+	}
+#endif	
 
-	void HandleExportCommand (object sender, EventArgs e)
+	private Gtk.Dialog info_display_window;
+	public void HandleInfoDisplayDestroy (object sender, EventArgs args)
+	{
+		info_display_window = null;
+		info_display = null;
+	}
+	
+	void HandleViewFullExif (object sender, EventArgs args)
+	{
+		if (info_display_window != null) {
+			info_display_window.Present ();
+			return;
+		}
+
+		info_display = new FSpot.InfoDisplay ();
+		info_display_window = new Gtk.Dialog ("EXIF Data", main_window, 
+						      Gtk.DialogFlags.NoSeparator | Gtk.DialogFlags.DestroyWithParent);
+		info_display_window.SetDefaultSize (400, 400);
+		Gtk.ScrolledWindow scroll = new ScrolledWindow ();
+		info_display_window.VBox.PackStart (scroll);
+		scroll.Add (info_display);
+
+		info_display.Photo = CurrentPhoto;
+	       
+		info_display_window.ShowAll ();
+		info_display_window.Destroyed += HandleInfoDisplayDestroy;
+	}
+
+
+#if false
+	void HandleExportCommand (object sender, EventArgs args)
 	{
 		ExportCommand.Gallery cmd = new ExportCommand.Gallery ();
 
@@ -847,8 +750,26 @@ public class MainWindow {
 			Console.WriteLine ("success");
 		}
 	}	
+#else
+	void HandleExportCommand (object sender, EventArgs args)
+	{
+		FSpot.GalleryExport export = new FSpot.GalleryExport (SelectedPhotos ());
+	}
+#endif
 
-	void HandleSendMailCommand (object sender, EventArgs e)
+#if false 
+	void HandleExportCommand (object sender, EventArgs args)
+	{
+		FotkiRemote fr = new FotkiRemote ();
+		//fr.Login ("mikey@gmail.com", "joey");
+		fr.Login ("mikey", "joey");
+		foreach (Photo p in SelectedPhotos ()) {
+			fr.Upload (p);
+		}
+	}
+#endif
+
+	void HandleSendMailCommand (object sender, EventArgs args)
 	{
 		StringBuilder url = new StringBuilder ("mailto:?subject=my%20photos");
 
@@ -861,7 +782,32 @@ public class MainWindow {
 		Gnome.Url.Show (url.ToString ());
 	}
 
-	void HandleArrangeByTime (object sender, EventArgs e)
+	void HandleAbout (object sender, EventArgs args)
+	{
+		string [] authors = new string [] {
+			"Ettore Perazzoli",
+			"Lawrence Ewing",
+			"Nat Friedman",
+			"Miguel de Icaza",
+			"Vladimir Vukicevic",
+			"Jon Trowbridge",
+			"Joe Shaw",
+			"Tambet Ingo",
+			"MOREAU Vincent",
+			"Lee Willis",
+			"Peter Johanson",
+			"Grahm Orr",
+			"Cheslack-Postava",
+			"Patanjali Somayaji",
+			"Matt Jones"
+		};
+			
+		Gnome.About about = new About ("F-Spot", "0.0.3", "Copyright 2003-2004 Novell Inc.", null, authors, null, null, null);
+
+		about.Show ();
+	}
+
+	void HandleArrangeByTime (object sender, EventArgs args)
 	{
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
 		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
@@ -870,7 +816,7 @@ public class MainWindow {
 		adaptor.GlassSet += HandleAdaptorGlassSet;
 	}
 
-	void HandleArrangeByDirectory (object sender, EventArgs e)
+	void HandleArrangeByDirectory (object sender, EventArgs args)
 	{
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
 		FSpot.GroupAdaptor adaptor = new FSpot.DirectoryAdaptor (query);		
@@ -892,10 +838,7 @@ public class MainWindow {
 		PhotoVersionCommands.Create cmd = new PhotoVersionCommands.Create ();
 
 		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
-			info_box.Update ();
-			photo_view.Update ();
-			icon_view.UpdateThumbnail (current_photo_idx);
-			UpdateMenus ();
+			UpdateViews (current_photo_idx);
 		}
 	}
 
@@ -904,10 +847,7 @@ public class MainWindow {
 		PhotoVersionCommands.Delete cmd = new PhotoVersionCommands.Delete ();
 
 		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
-			info_box.Update ();
-			photo_view.Update ();
-			icon_view.UpdateThumbnail (current_photo_idx);
-			UpdateMenus ();
+			UpdateViews (current_photo_idx);
 		}
 	}
 
@@ -931,28 +871,25 @@ public class MainWindow {
 		PhotoVersionCommands.Rename cmd = new PhotoVersionCommands.Rename ();
 
 		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
-			info_box.Update ();
-			photo_view.Update ();
-			icon_view.UpdateThumbnail (current_photo_idx);
-			UpdateMenus ();
+			UpdateViews (current_photo_idx);
 		}
 	}
 
-	void HandleCreateNewTagCommand (object sender, EventArgs args)
+	public void HandleCreateNewTagCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
 		if (command.Execute (TagCommands.TagType.Tag))
 			tag_selection_widget.Update ();
 	}
 
-	void HandleCreateNewCategoryCommand (object sender, EventArgs args)
+	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
 		if (command.Execute (TagCommands.TagType.Category))
 			tag_selection_widget.Update ();
 	}
 
-	void HandleAttachTagCommand (object obj, EventArgs args)
+	public void HandleAttachTagCommand (object obj, EventArgs args)
 	{
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
 		
@@ -962,27 +899,21 @@ public class MainWindow {
 	void AttachTags (Tag [] tags, int [] ids) 
 	{
 		foreach (int num in ids) {
-			Photo photo = query.Photos [num];
-			
-			AddTagExtended (photo, tags);
-			InvalidateViews (num);			
+			AddTagExtended (num, tags);
 		}
 	}
 
-	void HandleRemoveTagCommand (object obj, EventArgs args)
+	public void HandleRemoveTagCommand (object obj, EventArgs args)
 	{
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
 
 		foreach (int num in SelectedIds ()) {
-			Photo photo = query.Photos [num];
-			photo.RemoveTag (tags);
-			db.Photos.Commit (photo);
-			
-			InvalidateViews (num);
+			query.Photos [num].RemoveTag (tags);
+			query.Commit (num);
 		}
 	}
 
-	void HandleEditSelectedTag (object obj, EventArgs args)
+	public void HandleEditSelectedTag (object obj, EventArgs args)
 	{
 		Tag [] tags = tag_selection_widget.TagHighlight ();
 		if (tags.Length != 1)
@@ -1008,6 +939,16 @@ public class MainWindow {
 		icon_view.ThumbnailWidth = 256;	
 	}
 
+	void HandleDisplayTags (object sender, EventArgs args)
+	{
+		icon_view.DisplayTags = !icon_view.DisplayTags;
+	}
+	
+	void HandleDisplayDates (object sender, EventArgs args)
+	{
+		icon_view.DisplayDates = !icon_view.DisplayDates;
+	}
+
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
 #if true
@@ -1040,12 +981,14 @@ public class MainWindow {
 		
 		if (photos.Length == 0) {
 			Console.WriteLine ("No photos available -- no slideshow");
+			main_window.GdkWindow.Cursor = null;
 			return false;
 		}
 
 		SlideView slideview = new SlideView (bg, photos);
 		win.ButtonPressEvent += HandleSlideViewButtonPressEvent;
-		win.AddEvents ((int) EventMask.ButtonPressMask);
+		win.KeyPressEvent += HandleSlideViewKeyPressEvent;
+		win.AddEvents ((int) (EventMask.ButtonPressMask | EventMask.KeyPressMask));
 		win.Add (slideview);
 		win.Decorated = false;
 		win.Fullscreen();
@@ -1067,6 +1010,14 @@ public class MainWindow {
 		return false;
 	}
 	
+	[GLib.ConnectBefore]
+	private void HandleSlideViewKeyPressEvent (object sender, KeyPressEventArgs args)
+	{
+		Gtk.Window win = sender as Gtk.Window;
+		win.Destroy ();
+		args.RetVal = true;
+	}
+
 	private void HandleSlideViewButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
 		Gtk.Window win = sender as Gtk.Window;
@@ -1074,8 +1025,20 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
+	FSpot.FullScreenView fsview;
 	void HandleViewFullscreen (object sender, EventArgs args)
 	{
+		fsview = new FSpot.FullScreenView (query);
+		fsview.Show ();
+
+		// FIXME this needs to be another mode like PhotoView and IconView mode.
+
+		if (current_photo_idx != PHOTO_IDX_NONE)
+			fsview.View.CurrentPhoto = current_photo_idx;
+		else 
+			fsview.View.CurrentPhoto = 0;
+
+
 	}
 
 	void HandleZoomOut (object sender, EventArgs args)
@@ -1166,7 +1129,7 @@ public class MainWindow {
 		icon_view.UnselectAllCells ();
 	}
 	
-	void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
+	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
 	{
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
 		
@@ -1251,12 +1214,9 @@ public class MainWindow {
 	void UpdateForVersionIdChange (uint version_id)
 	{
 		CurrentPhoto.DefaultVersionId = version_id;
-		db.Photos.Commit (CurrentPhoto);
+		query.Commit (current_photo_idx);
 
-		info_box.Update ();
-		photo_view.Update ();
-		icon_view.UpdateThumbnail (current_photo_idx);
-		UpdateMenus ();
+		UpdateViews (current_photo_idx);
 	}
 
 	void HandleVersionIdChanged (PhotoVersionMenu menu)
@@ -1291,6 +1251,12 @@ public class MainWindow {
 		UpdateMenus ();
 	}
 
+	void HandleQueryItemChanged (PhotoQuery query, int item)
+	{
+		Photo photo = query.Photos [item];
+		if (info_box.Photo == photo)
+			info_box.Update ();
+	}
 	//
 	// Handle Main Menu 
 
