@@ -39,6 +39,7 @@ public class PhotoView : EventBox {
 		set {
 			query = value;
 			query.Reload += HandleQueryReload;
+			query.ItemChanged += HandleQueryItemChanged;
 
 			// FIXME which picture to display?
 			current_photo = 0;
@@ -46,9 +47,16 @@ public class PhotoView : EventBox {
 		}
 	}
 	
-	private void HandleQueryReload (PhotoQuery query) {
+	private void HandleQueryReload (PhotoQuery query) 
+	{
 		Update ();
 	}
+	
+	private void HandleQueryItemChanged (PhotoQuery query, int item)
+	{
+		if (item == current_photo)
+			Update ();
+	}
 
 	private FSpot.ImageView image_view;
 	private TagView tag_view;
@@ -56,6 +64,8 @@ public class PhotoView : EventBox {
 	private Label count_label;
 	private Entry description_entry;
 
+	FSpot.AsyncPixbufLoader loader = new FSpot.AsyncPixbufLoader ();
+
 	private const double MAX_ZOOM = 5.0;
 
 	private double zoom;
@@ -86,7 +96,6 @@ public class PhotoView : EventBox {
 	public event UpdateFinishedHandler UpdateFinished;
 
 	// Selection constraints.
-
 	private const string CONSTRAINT_RATIO_IDX_KEY = "FEditModeManager::constraint_idx";
 
 	private struct SelectionConstraint {
@@ -97,8 +106,6 @@ public class PhotoView : EventBox {
 	private OptionMenu constraints_option_menu;
 	private int selection_constraint_ratio_idx;
 
-	// FIXME: Should initialize statically, not sure how you do it.
-	// Maybe I should get myself a C# book at some point.
 	private static SelectionConstraint [] constraints;
 
 	private void HandleSelectionConstraintOptionMenuActivated (object sender, EventArgs args)
@@ -109,40 +116,6 @@ public class PhotoView : EventBox {
 
 	private OptionMenu CreateConstraintsOptionMenu ()
 	{
-		if (constraints == null) {
-			constraints = new SelectionConstraint [10];
-
-			constraints[0].Label = "No Constraint";
-			constraints[0].XyRatio = 0.0;
-
-			constraints[1].Label = "4 x 3 (Book)";
-			constraints[1].XyRatio = 4.0 / 3.0;
-
-			constraints[2].Label = "4 x 6 (Postcard)";
-			constraints[2].XyRatio = 6.0 / 4.0;
-
-			constraints[3].Label = "5 x 7 (L, 2L)";
-			constraints[3].XyRatio = 7.0 / 5.0;
-
-			constraints[4].Label = "8 x 10";
-			constraints[4].XyRatio = 10.0 / 8.0;
-
-			constraints[5].Label = "4 x 3 Portrait (Book)";
-			constraints[5].XyRatio = 3.0 / 4.0;
-
-			constraints[6].Label = "4 x 6 Portrait (Postcard)";
-			constraints[6].XyRatio = 4.0 / 6.0;
-
-			constraints[7].Label = "5 x 7 Portrait (L, 2L)";
-			constraints[7].XyRatio = 5.0 / 7.0;
-
-			constraints[8].Label = "8 x 10 Portrait";
-			constraints[8].XyRatio = 8.0 / 10.0;
-
-			constraints[9].Label = "Square";
-			constraints[9].XyRatio = 1.0;
-		}
-
 		Menu menu = new Menu ();
 
 		int i = 0;
@@ -162,8 +135,12 @@ public class PhotoView : EventBox {
 		return constraints_option_menu;
 	}
 
-
 	// Display.
+	private void HandlePixbufAreaUpdated (object sender, Gdk.AreaUpdatedArgs args)
+	{
+		// FIXME we should really only expose the updated region.
+		image_view.QueueDraw ();
+	}
 
 	private void UpdateImageView ()
 	{
@@ -171,19 +148,14 @@ public class PhotoView : EventBox {
 			try {
 				Pixbuf old = image_view.Pixbuf;
 				
-				image_view.Pixbuf = FSpot.PhotoLoader.Load (Query, current_photo);
-				tag_view.Current = Query.Photos [current_photo];
-				/*
-				** This is a hack seeing if the max size stuff acutally helps loading speed 
-				**
-				*/
-				/*
-				 if (image_view.Allocation.Width > 0 && image_view.Allocation.Height > 0)
-					image_view.Pixbuf = PixbufUtils.LoadAtMaxSize (Query.Photos [current_photo].DefaultVersionPath, 
-										       image_view.Allocation.Width, 
-										       image_view.Allocation.Height);
-				
-				*/
+				bool load_async = true;
+				if (load_async) {
+					image_view.Pixbuf = loader.Load (Query.Photos [current_photo].DefaultVersionPath);
+					loader.Loader.AreaUpdated += HandlePixbufAreaUpdated;
+					tag_view.Current = Query.Photos [current_photo];
+				} else {
+					image_view.Pixbuf = FSpot.PhotoLoader.Load (Query, current_photo);
+				}
 
 				if (old != null)
 					old.Dispose ();
@@ -195,7 +167,6 @@ public class PhotoView : EventBox {
 			image_view.Pixbuf = null;
 		}
 
-		//System.GC.Collect ();
 		image_view.UnsetSelection ();
 		UpdateZoom ();
 	}
@@ -212,10 +183,9 @@ public class PhotoView : EventBox {
 	}
 
 	private void UpdateZoom ()
-	{
-		const double EPSILON = 1e-6; // FIXME: Can we use something in corlib instead of this?
-		
+	{		
 		Pixbuf pixbuf = image_view.Pixbuf;
+
 		if (pixbuf == null)
 			return;
 
@@ -228,12 +198,12 @@ public class PhotoView : EventBox {
 		double image_zoom = (MAX_ZOOM - zoom_to_fit) * Zoom + zoom_to_fit;
 		Console.WriteLine ("Zoom {2} zoom_to_fit {0} image_zoom {1}", zoom_to_fit, image_zoom, Zoom);
 
-		if (Math.Abs (Zoom) < EPSILON)
+		if (Math.Abs (Zoom) < double.Epsilon)
 			((ScrolledWindow) image_view.Parent).SetPolicy (PolicyType.Never, PolicyType.Never);
 
 		image_view.SetZoom (image_zoom, image_zoom);
 
-		if (Math.Abs (Zoom) < EPSILON && restore_scrollbars_idle_id == 0)
+		if (Math.Abs (Zoom) < double.Epsilon && restore_scrollbars_idle_id == 0)
 			restore_scrollbars_idle_id = Idle.Add (new IdleHandler (IdleUpdateScrollbars));
 	}
 
@@ -260,6 +230,7 @@ public class PhotoView : EventBox {
 
 	private void UpdateDescriptionEntry ()
 	{
+		description_entry.Changed -= HandleDescriptionChanged;
 		if (Query.Photos.Length > 1 && current_photo < Query.Photos.Length) {
 			description_entry.Sensitive = true;
 			description_entry.Text = Query.Photos[current_photo].Description;
@@ -267,6 +238,7 @@ public class PhotoView : EventBox {
 			description_entry.Sensitive = false;
 			description_entry.Text = "";
 		}
+		description_entry.Changed += HandleDescriptionChanged;
 	}    
 
 	public void Update ()
@@ -316,10 +288,12 @@ public class PhotoView : EventBox {
 	private void HandleImageViewKeyPressEvent (object sender, KeyPressEventArgs args)
 	{
 		switch (args.Event.Key) {
-		case Gdk.Key.Left:
+		case Gdk.Key.Page_Up:
+		case Gdk.Key.KP_Page_Up:
 			HandleDisplayPreviousButtonClicked (sender, null);
 			break;
-		case Gdk.Key.Right:
+		case Gdk.Key.Page_Down:
+		case Gdk.Key.KP_Page_Down:
 			HandleDisplayNextButtonClicked (sender, null);
 			break;
 		default:
@@ -357,28 +331,39 @@ public class PhotoView : EventBox {
 		int x, y, width, height;
 		if (! image_view.GetSelection (out x, out y, out width, out height))
 			return;
-
-		Photo photo = query.Photos [CurrentPhoto];
-		if (photo.DefaultVersionId == Photo.OriginalVersionId) {
-			photo.DefaultVersionId = photo.CreateDefaultModifiedVersion (photo.DefaultVersionId, false);
-			photo_store.Commit (photo);
+		
+		Pixbuf original_pixbuf = image_view.Pixbuf;
+		if (original_pixbuf == null) {
+			Console.WriteLine ("No image");
+			return;
 		}
 
-		Pixbuf original_pixbuf = image_view.Pixbuf;
+		Photo photo = query.Photos [CurrentPhoto];
+
 		Pixbuf cropped_pixbuf = new Pixbuf (original_pixbuf.Colorspace, false, original_pixbuf.BitsPerSample,
 						    width, height);
 
-		original_pixbuf.CopyArea (x, y, width, height, cropped_pixbuf, 0, 0);
-		
-		image_view.Pixbuf = cropped_pixbuf;
+		original_pixbuf.CopyArea (x, y, width, height, cropped_pixbuf, 0, 0);		
 
 		// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
 		// be fixed there.
+		image_view.Pixbuf = cropped_pixbuf;
 		image_view.UnsetSelection ();
 
+		System.Console.WriteLine ("Got here");
+
 		try {
-			cropped_pixbuf.Savev (photo.DefaultVersionPath, "jpeg", null, null);
-			PhotoStore.GenerateThumbnail (photo.DefaultVersionPath);
+			if (photo.DefaultVersionId == Photo.OriginalVersionId) {
+				photo.DefaultVersionId = photo.CreateDefaultModifiedVersion (photo.DefaultVersionId, false);
+				cropped_pixbuf.Savev (photo.DefaultVersionPath, "jpeg", null, null);
+				PhotoStore.GenerateThumbnail (photo.DefaultVersionPath);
+				query.Commit (CurrentPhoto);
+			} else {
+				// FIXME we need to invalidate the thumbnail in the cache as well
+				cropped_pixbuf.Savev (photo.DefaultVersionPath, "jpeg", null, null);
+				PhotoStore.GenerateThumbnail (photo.DefaultVersionPath);
+				query.MarkChanged (CurrentPhoto);
+			}
 		} catch (GLib.GException ex) {
 			// FIXME error dialog.
 			Console.WriteLine ("error {0}", ex);
@@ -391,17 +376,17 @@ public class PhotoView : EventBox {
 	}
 
 	private void HandleUnsharpButtonClicked (object sender, EventArgs args) {
-		image_view.Pixbuf = PixbufUtils.UnsharpMask (image_view.Pixbuf, 6, 2, 0);
+		//image_view.Pixbuf = PixbufUtils.UnsharpMask (image_view.Pixbuf, 6, 2, 0);
+		//image_view.Pixbuf = PixbufUtils.ColorCorrect (image_view.Pixbuf);
+		new ColorDialog (Query, current_photo, image_view.Pixbuf);
 	}	
 
 	private void HandleDescriptionChanged (object sender, EventArgs args) {
 		if (!CurrentPhotoValid ())
 			return;
 
-		Photo photo = Query.Photos[current_photo];
-
-		photo.Description = description_entry.Text;
-		photo_store.Commit (photo);
+		Query.Photos[current_photo].Description = description_entry.Text;
+		Query.Commit (current_photo);
 	}
 
 	// Constructor.
@@ -415,6 +400,41 @@ public class PhotoView : EventBox {
 		}
 	}
 
+	static PhotoView ()
+	{
+		constraints = new SelectionConstraint [10];
+
+		constraints[0].Label = "No Constraint";
+		constraints[0].XyRatio = 0.0;
+
+		constraints[1].Label = "4 x 3 (Book)";
+		constraints[1].XyRatio = 4.0 / 3.0;
+
+		constraints[2].Label = "4 x 6 (Postcard)";
+		constraints[2].XyRatio = 6.0 / 4.0;
+
+		constraints[3].Label = "5 x 7 (L, 2L)";
+		constraints[3].XyRatio = 7.0 / 5.0;
+
+		constraints[4].Label = "8 x 10";
+		constraints[4].XyRatio = 10.0 / 8.0;
+
+		constraints[5].Label = "4 x 3 Portrait (Book)";
+		constraints[5].XyRatio = 3.0 / 4.0;
+
+		constraints[6].Label = "4 x 6 Portrait (Postcard)";
+		constraints[6].XyRatio = 4.0 / 6.0;
+
+		constraints[7].Label = "5 x 7 Portrait (L, 2L)";
+		constraints[7].XyRatio = 5.0 / 7.0;
+
+		constraints[8].Label = "8 x 10 Portrait";
+		constraints[8].XyRatio = 8.0 / 10.0;
+
+		constraints[9].Label = "Square";
+		constraints[9].XyRatio = 1.0;
+	}
+
 	public PhotoView (PhotoStore photo_store)
 		: base ()
 	{
@@ -425,15 +445,29 @@ public class PhotoView : EventBox {
 		Box vbox = new VBox (false, 6);
 		Add (vbox);
 
+		EventBox eventbox = new EventBox ();
 		Frame frame = new Frame ();
+		eventbox.Add (frame);
 		frame.ShadowType = ShadowType.In;
-		vbox.PackStart (frame, true, true, 0);
+		vbox.PackStart (eventbox, true, true, 0);
 		
 		Box inner_vbox = new VBox (false , 2);
-		frame.Add (inner_vbox);
 
+		frame.Add (inner_vbox);
+		
 		image_view = new FSpot.ImageView ();
+
 		ScrolledWindow image_view_scrolled = new ScrolledWindow (null, null);
+
+		Gdk.Color color = eventbox.Style.Background (Gtk.StateType.Normal);
+		color.Red = (ushort) (color.Red / 2);
+		color.Blue = (ushort) (color.Blue / 2);
+		color.Green = (ushort) (color.Green / 2);
+
+		image_view.ModifyBg (Gtk.StateType.Normal, color);
+		eventbox.ModifyBg (Gtk.StateType.Normal, color);
+		image_view_scrolled.ModifyBg (Gtk.StateType.Normal, color);
+
 		image_view_scrolled.SetPolicy (PolicyType.Automatic, PolicyType.Automatic);
 		image_view_scrolled.ShadowType = ShadowType.None;
 		image_view_scrolled.Add (image_view);
@@ -451,7 +485,7 @@ public class PhotoView : EventBox {
 
 		description_entry = new Entry ();
 		inner_hbox.PackStart (description_entry, true, true, 0);
-		description_entry.Changed += new EventHandler (HandleDescriptionChanged);
+		description_entry.Changed += HandleDescriptionChanged;
 		
 		inner_vbox.PackStart (inner_hbox, false, true, 0);
 
