@@ -6,6 +6,9 @@ using System.IO;
 
 class PixbufUtils {
 
+	public static Pixbuf ErrorPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-question-mark.png");
+	public static Pixbuf LoadingPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-loading.png");
+
 	public static int GetSize (Pixbuf pixbuf)
 	{
 		return Math.Max (pixbuf.Width, pixbuf.Height);
@@ -122,6 +125,7 @@ class PixbufUtils {
 	{
 		double scale = Math.Min  (width / (double)pixbuf.Width, height / (double)pixbuf.Height);
 	
+		
 		int scale_width = (int)(scale * pixbuf.Width);
 		int scale_height = (int)(scale * pixbuf.Height);
 		return pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
@@ -233,8 +237,55 @@ class PixbufUtils {
  		Gdk.Pixbuf ret = (Gdk.Pixbuf) GLib.Object.GetObject(raw_ret, true);
 		return ret;
 	}	
+	
+	public static unsafe Pixbuf ColorAdjust (Pixbuf src, double brightness, double contrast,
+					  double hue, double saturation, int src_color, int dest_color)
+	{
+		Pixbuf adjusted = new Pixbuf (Colorspace.Rgb, src.HasAlpha, 8, src.Width, src.Height);
+		ColorAdjust (src, adjusted, brightness, contrast, hue, saturation, src_color, dest_color);
+		return adjusted;
+	}
 
-#if STUFF_WE_HAVE_TO_RESTORE
+	public static Cms.Format PixbufCmsFormat (Pixbuf buf)
+	{
+		return buf.HasAlpha ? Cms.Format.Rgba8 : Cms.Format.Rgb8;
+	}
+
+
+	public static unsafe void ColorAdjust (Pixbuf src, Pixbuf dest, 
+					       double brightness, double contrast,
+					       double hue, double saturation, 
+					       int src_color, int dest_color)
+	{
+		if (src.Width != dest.Width || src.Height != dest.Height)
+			throw new Exception ("Invalid Dimensions");
+
+		//Cms.Profile eos10d = new Cms.Profile ("/home/lewing/ICCProfiles/EOS-10D-True-Color-Non-Linear.icm");
+		Cms.Profile srgb = Cms.Profile.CreateSRgb ();
+		Cms.Profile bchsw = Cms.Profile.CreateAbstract (10, brightness, contrast,
+								hue, saturation, src_color, 
+								dest_color);
+
+		Cms.Profile [] list = new Cms.Profile [] { srgb, bchsw, srgb };
+		Cms.Transform trans = new Cms.Transform (list, 
+							 PixbufCmsFormat (src),
+							 PixbufCmsFormat (dest),
+							 Cms.Intent.Perceptual, 0x0100);
+
+		int width = src.Width;
+		byte * srcpix  = (byte *) src.Pixels;
+		byte * destpix = (byte *) dest.Pixels;
+
+		for (int row = 0; row < src.Height; row++) {
+			trans.Apply ((IntPtr) (srcpix + row * src.Rowstride),
+				     (IntPtr) (destpix + row * dest.Rowstride), 
+				     (uint)width);
+		}
+		
+		trans.Dispose ();
+		srgb.Dispose ();
+		bchsw.Dispose ();
+	}
 
 	// Bindings from libf.
 
@@ -274,6 +325,4 @@ class PixbufUtils {
 			throw new GLib.GException (error);
 		}
 	}
-
-#endif
 }
