@@ -38,7 +38,7 @@ class Semaphore {
 	}
 }
 
-class UriList : ArrayList {
+public class UriList : ArrayList {
 	public UriList (Photo [] photos) {
 		foreach (Photo p in photos) {
 			Uri uri;
@@ -76,10 +76,25 @@ class UriList : ArrayList {
 		}
 	}
 
+	public UriList (string [] uris)
+	{	
+		// FIXME this is so lame do real chacking at some point
+		foreach (string str in uris) {
+			Uri uri;
+
+			if (str.StartsWith ("/"))
+				uri = new Uri ("file://" + str);
+			else 
+				uri = new Uri (str);
+			
+			Add (uri);
+		}
+	}
+
 	public UriList (string data) {
 		LoadFromString (data);
 	}
-
+	
 	public UriList (Gtk.SelectionData selection) 
 	{
 		// FIXME this should check the atom etc.
@@ -132,6 +147,17 @@ class GtkUtil {
 		i.Show ();
 	}
 	
+	public static void MakeMenuItem (Gtk.Menu menu, string label, string image_name, EventHandler e, bool enabled)
+	{
+		Gtk.ImageMenuItem i = new Gtk.ImageMenuItem (label);
+		i.Activated += e;
+                i.Sensitive = enabled;
+		i.Image = new Gtk.Image (image_name, Gtk.IconSize.Menu);
+		
+		menu.Append (i);
+		i.Show ();
+	}
+
 	public static void MakeMenuSeparator (Gtk.Menu menu)
 	{
 		Gtk.SeparatorMenuItem i = new Gtk.SeparatorMenuItem ();
